@@ -5,7 +5,10 @@ import numpy as np
 import torch.nn as nn
 import torch.nn.functional as F
 
+from abc import abstractmethod
+from abc import ABC
 from PIL import Image
+from enum import Enum
 from tqdm import tqdm
 from torch import Tensor
 from typing import Any
@@ -30,6 +33,9 @@ from .common import restrict_wh
 from .common import get_suitable_size
 from .common import APIMixin
 from .common import ReadImageResponse
+from ..third_party import MiDaSAPI
+from ..third_party import MLSDDetector
+from ..third_party import OpenposeDetector
 from ....zoo import DLZoo
 from ....data import predict_tensor_data
 from ....data import TensorInferenceData
@@ -56,6 +62,11 @@ from ....models.cv.diffusion.samplers.ddim import DDIMMixin
 from ....models.cv.diffusion.samplers.solver import DPMSolver
 from ....models.cv.diffusion.samplers.k_samplers import KSamplerMixin
 
+try:
+    import cv2
+except:
+    cv2 = None
+
 
 class switch_sampler_context:
     def __init__(self, api: "DiffusionAPI", sampler: Optional[str]):
@@ -92,6 +103,7 @@ class MaskedCond(NamedTuple):
 
 
 T = TypeVar("T", bound="DiffusionAPI")
+TAnnotator = TypeVar("TAnnotator", bound="Annotator")
 
 
 class DiffusionAPI(APIMixin):
@@ -1095,9 +1107,108 @@ class DiffusionAPI(APIMixin):
             )
 
 
+class ControlNetHints(str, Enum):
+    DEPTH = "depth"
+    CANNY = "canny"
+    POSE = "post"
+    MLSD = "mlsd"
+
+
+class Annotator(ABC):
+    @abstractmethod
+    def __init__(self, device: torch.device) -> None:
+        pass
+
+    @abstractmethod
+    def to(self: TAnnotator, device: torch.device, *, use_half: bool) -> TAnnotator:
+        pass
+
+    @abstractmethod
+    def annotate(self, uint8_rgb: np.ndarray, **kwargs: Any) -> np.ndarray:
+        pass
+
+
+class DepthAnnotator(Annotator):
+    def __init__(self, device: torch.device) -> None:
+        self.m = MiDaSAPI(device)
+
+    def to(self, device: torch.device, *, use_half: bool) -> "DepthAnnotator":
+        if use_half:
+            self.m.model.half()
+        else:
+            self.m.model.float()
+        self.m.model.to(device)
+        return self
+
+    def annotate(self, uint8_rgb: np.ndarray) -> np.ndarray:  # type: ignore
+        return self.m.detect_depth(uint8_rgb)
+
+
+class CannyAnnotator(Annotator):
+    def __init__(self, device: torch.device) -> None:
+        if cv2 is None:
+            raise ValueError("`cv2` is needed for `CannyAnnotator`")
+
+    def to(self, device: torch.device, *, use_half: bool) -> "CannyAnnotator":
+        return self
+
+    def annotate(  # type: ignore
+        self,
+        uint8_rgb: np.ndarray,
+        *,
+        low_threshold: int,
+        high_threshold: int,
+    ) -> np.ndarray:
+        return cv2.Canny(uint8_rgb, low_threshold, high_threshold)
+
+
+class PoseAnnotator(Annotator):
+    def __init__(self, device: torch.device) -> None:
+        self.m = OpenposeDetector(device)
+
+    def to(self, device: torch.device, *, use_half: bool) -> "PoseAnnotator":
+        self.m.to(device, use_half=use_half)
+        return self
+
+    def annotate(self, uint8_rgb: np.ndarray) -> np.ndarray:  # type: ignore
+        return self.m(uint8_rgb)[0]
+
+
+class MLSDAnnotator(Annotator):
+    def __init__(self, device: torch.device) -> None:
+        self.m = MLSDDetector(device)
+
+    def to(self, device: torch.device, *, use_half: bool) -> "MLSDAnnotator":
+        self.m.to(device, use_half=use_half)
+        return self
+
+    def annotate(  # type: ignore
+        self,
+        uint8_rgb: np.ndarray,
+        *,
+        value_threshold: float,
+        distance_threshold: float,
+    ) -> np.ndarray:
+        return self.m(uint8_rgb, value_threshold, distance_threshold)
+
+
 class ControlledDiffusionAPI(DiffusionAPI):
-    current: Optional[str]
+    current: Optional[ControlNetHints]
     weights: tensor_dict_type
+    annotators: Dict[ControlNetHints, Annotator] = {}
+
+    defaults = {
+        ControlNetHints.DEPTH: "ldm.sd_v1.5.control.depth",
+        ControlNetHints.CANNY: "ldm.sd_v1.5.control.canny",
+        ControlNetHints.POSE: "ldm.sd_v1.5.control.pose",
+        ControlNetHints.MLSD: "ldm.sd_v1.5.control.mlsd",
+    }
+    annotator_classes: Dict[ControlNetHints, Type[Annotator]] = {
+        ControlNetHints.DEPTH: DepthAnnotator,
+        ControlNetHints.CANNY: CannyAnnotator,
+        ControlNetHints.POSE: PoseAnnotator,
+        ControlNetHints.MLSD: MLSDAnnotator,
+    }
 
     def __init__(
         self,
@@ -1128,29 +1239,71 @@ class ControlledDiffusionAPI(DiffusionAPI):
         use_half: bool = False,
     ) -> None:
         super().to(device, use_amp=use_amp, use_half=use_half)
-        for d in self.weights.values():
+        for hint, d in self.weights.items():
+            # weights
             for k, v in d.items():
                 v = v.half() if use_half else v.float()
                 d[k] = v.to(device)
+            # annotator
+            annotator = self.annotators.get(hint)
+            if annotator is not None:
+                self.annotators[hint] = annotator.to(device, use_half=use_half)
 
-    def prepare(self, names2tags: Dict[str, str]) -> None:
-        for name, tag in names2tags.items():
-            self.weights[name] = torch.load(download_model(tag))
+    @property
+    def available(self) -> List[ControlNetHints]:
+        return list(self.weights)
 
-    def switch(self, name: str) -> None:
+    def prepare(self, hints2tags: Dict[ControlNetHints, str]) -> None:
+        for hint, tag in hints2tags.items():
+            self.weights[hint] = torch.load(download_model(tag))
+
+    def prepare_defaults(self) -> None:
+        self.prepare(self.defaults)
+
+    def switch(self, hint: ControlNetHints) -> None:
         if self.m.control_model is None:
             raise ValueError("`control_model` is not built yet")
-        if name == self.current:
+        if hint == self.current:
             return
-        d = self.weights.get(name)
+        d = self.weights.get(hint)
         if d is None:
             raise ValueError(
-                f"cannot find weights called '{name}', "
-                f"available weights are: {', '.join(self.weights.keys())}"
+                f"cannot find weights called '{hint}', "
+                f"available weights are: {', '.join(self.available)}"
             )
-        self.current = name
+        self.current = hint
         self.m.control_model.load_state_dict(d)
 
+    def prepare_annotator(self, hint: ControlNetHints) -> None:
+        if hint not in self.annotators:
+            annotator_class = self.annotator_classes.get(hint)
+            if annotator_class is None:
+                raise ValueError(f"annotator for '{hint}' is not implemented")
+            annotator = annotator_class(self.device)
+            annotator = annotator.to(self.device, use_half=self.use_half)
+            self.annotators[hint] = annotator
+
+    def prepare_annotators(self) -> None:
+        for hint in self.weights:
+            self.prepare_annotator(hint)
+
+    def get_hint(self, uint8_rgb: np.ndarray, **kwargs: Any) -> np.ndarray:
+        if self.current is None:
+            raise ValueError("`hint` is not specified yet, please call `switch` first.")
+        annotator = self.annotators.get(self.current)
+        if annotator is None:
+            raise ValueError(
+                f"annotator for '{self.current}' is not prepared yet, "
+                "please call `prepare_annotator`/`prepare_annotators` first."
+            )
+        kwargs[uint8_rgb] = uint8_rgb
+        out = safe_execute(annotator.annotate, kwargs)
+        if len(out.shape) == 2:
+            out = out[..., None]
+        if out.shape[-1] == 1:
+            out = np.repeat(out, 3, axis=2)
+        return out
+
 
 def _ldm(
     model: str,
@@ -1316,5 +1469,6 @@ def ldm_semantic(pretrained: bool = True) -> DLPipeline:
 
 __all__ = [
     "DiffusionAPI",
+    "ControlNetHints",
     "ControlledDiffusionAPI",
 ]
