@@ -17,6 +17,7 @@ from typing import NamedTuple
 from cftool.misc import print_warning
 from cftool.misc import Saving
 from cftool.misc import WithRegister
+from cftool.array import to_numpy
 from cftool.array import to_device
 from cftool.types import tensor_dict_type
 from torch.utils.data import Dataset
@@ -28,6 +29,7 @@ from torch.utils.data.distributed import DistributedSampler
 from ..types import sample_weights_type
 from ..schema import IDataset
 from ..schema import IDataLoader
+from ..schema import IDataModule
 from ..misc.toolkit import get_ddp_info
 from ..misc.toolkit import get_world_size
 
@@ -38,7 +40,7 @@ except:
 
 
 data_modules: Dict[str, Type["DataModule"]] = {}
-DataModuleType = TypeVar("DataModuleType", bound="DataModule", covariant=True)
+TDataModule = TypeVar("TDataModule", bound="DataModule", covariant=True)
 
 
 class BaseResponse(NamedTuple):
@@ -51,7 +53,7 @@ class LoadInfoResponse(NamedTuple):
     data_module_bytes: Optional[bytes]
 
 
-class DataModule(WithRegister[DataModuleType], metaclass=ABCMeta):
+class DataModule(IDataModule, WithRegister[TDataModule]):
     d = data_modules  # type: ignore
 
     id_file = "id.txt"
@@ -59,19 +61,6 @@ class DataModule(WithRegister[DataModuleType], metaclass=ABCMeta):
     data_folder = "data"
     package_folder = "data_module"
 
-    # inherit
-
-    @property
-    @abstractmethod
-    def info(self) -> Dict[str, Any]:
-        pass
-
-    def prepare(self, sample_weights: sample_weights_type) -> None:
-        pass
-
-    def initialize(self) -> Any:
-        pass
-
     # internal
 
     @abstractmethod
@@ -249,6 +238,7 @@ class DLLoader(IDataLoader):
             )
         super().__init__(sample_weights=sample_weights)
         self.loader = loader
+        self.use_numpy = False
         self.data = loader.dataset  # type: ignore
         self.batch_callback = batch_callback
         self.sampler_backup = loader.sampler
@@ -294,10 +284,14 @@ class DLLoader(IDataLoader):
 
         if self.batch_callback is not None:
             batch = self.batch_callback(batch)
-        if self.stream is not None:
+        if not self.use_numpy:
             with torch.cuda.stream(self.stream):
                 batch = to_device(batch, self.device, non_blocking=True)
-        batch = to_device(batch, self.device, non_blocking=True)
+        else:
+            batch = {
+                k: v if not isinstance(v, torch.Tensor) else to_numpy(v)
+                for k, v in batch.items()
+            }
         self.next_batch = batch
 
     @property
