@@ -68,7 +68,7 @@ class ViTEncoder(Encoder1DFromPatches):
             norm_kwargs=norm_kwargs,
             embedding_norm=embedding_norm,
             residual_after_norm=residual_after_norm,
-            feedforward_dim_ratio=feedforward_dim_ratio,
+            latent_dim_ratio=feedforward_dim_ratio,
             head_pooler=head_pooler,
             use_head_token=use_head_token,
             use_positional_encoding=use_positional_encoding,
@@ -143,7 +143,7 @@ class ViTEncoder2D(Encoder2DFromPatches):
             norm_kwargs=norm_kwargs,
             embedding_norm=embedding_norm,
             residual_after_norm=residual_after_norm,
-            feedforward_dim_ratio=feedforward_dim_ratio,
+            latent_dim_ratio=feedforward_dim_ratio,
             head_pooler=None,
             use_head_token=False,
             use_positional_encoding=use_positional_encoding,
