@@ -3,7 +3,7 @@ from typing import Dict
 from typing import Callable
 from cftool.types import tensor_dict_type
 
-from .protocol import IDLZooModelLoader
+from .schema import IDLZooModelLoader
 
 
 def get_callback(use_vision: bool, use_text: bool) -> Callable:
